import Backup.*;
import java.util.Vector;
import java.io.FilenameFilter;
import java.io.*;
public class AgentImpl extends AgentPOA {

  protected String name, password="";
  protected Vector sids = new Vector();
  
  public AgentImpl(String name, String password) {
    this(name);
    this.password = password;
  }
  
  public AgentImpl(String name) {
    this.name = name;
  }
  
  public String name() {
    return name;
  }
  
  public boolean checkPassword (String password) {
    return this.password.equals(password);
  }

  public void startSession(String password, String sid) 
                                       throws InvalidPWException {
    if ( ! checkPassword(password) ) {
      Global.LogMessage("Agent.checkPassword()", "Invalid password for " +
                                                 this.name +": " + password);
      throw new InvalidPWException();
                                   
    }				   
    sids.addElement(sid);
  }
 public void endSession(String sid) throws IDException {
   if ( sids.indexOf(sid) == -1) {
     Global.LogMessage("Agent.endSession()",
                        "endSession: SID " + sid + " not found.");             
     throw new IDException();
   } else {
     sids.removeElement(sid);
   }
 }

 public FileChunker getFile(String filename) throws FileException {
     FileChunkerImpl fci;
     try {
       fci=new FileChunkerImpl(filename);
       return FileChunkerHelper.narrow(
           _default_POA().servant_to_reference(fci)
       );
     } catch (Exception e) {
       
       Global.LogMessage("Agent.endSession()","agent.getFile(" + filename + 
                                                                  ") failed");
       throw new FileException(); 
     }
 }
 
 public FileIterator sendFiles(String sid, String[] files) 
                                             throws IDException {
   if ( sids.indexOf(sid) == -1) {
     throw new IDException(/*"endSession: SID " + sid + " not found."*/);
   } else {
     FileIteratorImpl fii = new FileIteratorImpl
                              ((new FileMaskExpander()).expandMask(files));
     try {
       return FileIteratorHelper.narrow(
                     _default_POA().servant_to_reference(fii)
                );
     } catch (Exception e) { 
       throw new IDException();
     }
   } 
 }
 
 public void restoreFiles (Storage stor, String sid, 
                           String[] files, boolean replace) 
               throws TransferFailedException {
   FileIterator fi;
   try {
     fi = stor.sendFiles(sid, files);
   } catch (Exception e) {
     Global.LogMessage("Agent.restore()", "Exception: " + e.getMessage());
     throw new TransferFailedException();
   }
   files:while ( fi.hasMoreFiles() ) {
     FileChunker fc;  
     try {
       fc=fi.nextFile();
     } catch (Exception e) { 
       // chybi reseni retries
       try {
         fc=fi.retryFile();
       } catch (Exception f) {
	 Global.LogMessage("Agent_"+this.name, "nextFile failed " + e); 
         continue;  
       }
     }
     String name=fc.fInfo().name; 
     File fil=new File(name);

     if ( fil.exists() ){
     // No replace
       if (! replace) {
         Global.LogMessage("Agent_"+this.name, "Skipping " + name);
         continue;
       }
     } else {
       try {
         File f1=new File(fil.getParent());
	 if ( f1 != null ) {
	   f1.mkdirs();
	 }
       } catch (SecurityException e) {
         Global.LogMessage("Agent_"+this.name, "Mkdirs failed for " + name);
       }
     }
     
     OutputStream out;
     try { 
       out = new FileOutputStream(name);
     } catch (IOException e) {
       Global.LogMessage("Agent_"+this.name, "Can\'t write " + name);
       continue;
     }
     while (fc.hasMoreChunks()) {
       byte [] chunk;
       try {
         chunk=fc.nextChunk();		//NextChunk
       } 
       catch (FileException e){
         // chybi reseni retries
         try {
           chunk=fc.retryChunk(); 	//Retry
	 } catch (BeforeFirstException f) {
	   e.printStackTrace();
	   continue;
	 }
       }
       catch (BeyondLastException f) {
         f.printStackTrace();
	 continue;
       }
       catch (Exception f) {
	 Global.LogMessage("Agent_"+this.name, "nextChunk failed " + f); 
	 try {
           out.close();
	   new File(name).delete();
	 } catch (IOException g) {}
	 continue files;  
       }
       try {
         out.write(chunk); 		//Zapis
       } catch (IOException e) {
	 Global.LogMessage("Agent_"+this.name, "can\'t write "+name +": " + e); 
	 try {
           out.close();
	   new File(name).delete();
	 } catch (IOException g) {}
	 continue files;  
       }
     }
   }
   try {
     stor.endSession(sid);
     fi._release();
   } catch (Exception e) {
     Global.LogMessage("restoreFiles()", "Error when endSession");
   }
 }
 
}
