import Backup.*;
// Server.java
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.PortableServer.*;
import java.io.*;

public class AgentServer {
  public static String name = "Agent";

    private static void printUsage() {

        System.err.println("usage: java AgentServer (options)");
        System.err.println();
        System.err.println("options:");
        System.err.println("  -name name  Specify agent name.");
        System.err.println("  -pass pass  Specify agents password.");
        System.err.println("  -h       This help screen.");

    } // printUsage()

  private static String readText() throws IOException{
    StringBuffer str = new StringBuffer();
    DataInputStream in = new DataInputStream(System.in);
    char c;
    c= (char) System.in.read();	
    while (c != '\n'){
      str.append(c);
      c= (char) System.in.read();	
//      System.out.print((int)c);	
    }
    return str.toString();		
  }

  public static void main(String[] argv) {
    String agentName = null;
    String password = null;	

    if (argv.length != 0) {

        // check parameters
        for (int i = 0; i < argv.length; i++) {
            String arg = argv[i];

            // options
            if (arg.startsWith("-")) {
                if (arg.equals("-name")) {
                    if (i == argv.length - 1) {
                        System.err.println("error: missing agent name");
                        System.exit(1);
                    }
                    agentName = argv[++i];
                    continue;
                }
                if (arg.equals("-pass")) {
                    if (i == argv.length - 1) {
                        System.err.println("error: missing password");
                        System.exit(1);
                    }
                    password = argv[++i];
                    continue;
                }
                if (arg.equals("-nopass")) {
                    password = "";
                    continue;
                }
                if (arg.equals("-h")) {
                    printUsage();
                    System.exit(1);
                }
            }
        }
    } 
    
    while (agentName == null || agentName=="") {
      System.out.print("Agent name: ");
      try {
        agentName = readText();
      }
      catch (IOException e) {
        System.err.println("Error by reading from System.in\n" + e);
      }
      System.out.print(agentName);
 	
    }
    if (password == null) {
      System.out.print("Password: ");
      try {
        password = readText();
      }
      catch (IOException e) {
        System.err.println("Error by reading from System.in\n" + e);
      }
    }



    try {
      // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(argv,null);

      // get a reference to the root POA
      POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

      // Create policies for our persistent POA
      org.omg.CORBA.Policy[] policies = {
        rootPOA.create_lifespan_policy(LifespanPolicyValue.TRANSIENT)
      };
      // Create myPOA with the right policies
      POA myPOA = rootPOA.create_POA( "myAgentPOA", rootPOA.the_POAManager(),
                                        policies );

      // Create the servant
      AgentImpl _Agent = new AgentImpl(agentName,password);

      // Activate the servant with the ID on myPOA
      myPOA.activate_object(_Agent);

      // Activate the POA manager
      rootPOA.the_POAManager().activate();


       org.omg.CORBA.Object ref=myPOA.servant_to_reference(_Agent); 
/* 
       
       FileWriter fw=new FileWriter("_Agent.ref");
       fw.write(orb.object_to_string(ref));
       fw.close();

      System.out.println("Object reference stored to file\n");
*/
      if (!Global.Initialize(orb)) {
         System.out.println("\nInit of NameService failed - "
                         +"it may not be running where you specified,\n"
                         +"or you didn't give the argument -ORBservice "
                         +"NameService <loc>,\nwhere <loc> is either IOR or "
                         +"corbaloc::host:port/NameService"                   );
         System.exit(1);
      }
      
      name += "_" + agentName;  
      try {
        if ( ! Global.bind(name, ref) ) {
          System.err.println(name + ": Can't bind to NamingService");
        }
      } catch (AlreadyBound e) {
         org.omg.CORBA.Object oldref=Global.Get(name);
        try {
          boolean result = oldref._non_existent();
          if (result) {
            throw new Exception();
          } else {
             System.err.println("Can't start: Different instance "
                                + "of " + name + " is running"           );
             System.exit(1);
          }
        } catch (Exception g) {
          try {
            NameComponent [] nc ={new NameComponent(name," ")};
            Global.GetNaming().unbind(nc);
            Global.bind(name, ref);
          } catch (Exception f) {
            System.out.println(name +": Can't bind to NamingService: " +f);
            System.exit(1);
          }
        }
      }
      
      Global.LogMessage("AgentServer.main()", "Agent " + name + " started.");
      
      // Wait for incoming requests
      orb.run();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

}
