import Backup.*;
import java.io.*;
import java.util.*;
import org.omg.CORBA.*;
import Backup.RepeatFreqPackage.Freq;
public class ControlImpl extends ControlPOA implements Runnable {
  
  /**
      Sleep for 1 minute before going futher.
  */
  public static final long SLEEP = 60000; 

  protected static SIDUtil sidgen = new SIDUtil('b');
   
  /**
      Minimal time for reschedule (you can't go bellow this)
  */
  protected int minMinutes=15;
  
  /**
      Retry time - 6 hours by default
  */
  protected RepeatFreq retry = new RepeatFreq((short)6,Freq.Hours);
  
  /** 
     Retry multiplificator - how many times the backup frequency must be
     greater than the retry time to reschedule a rule for retry after
     a failure, caused possibly by missing agent, etc.
  */
  protected int retry_mul = 2;
  /** 
     Stop running the control unit when false
  */
  protected boolean running = true;

  protected Hashtable aInfos; 
  protected Vector rules;
  
  protected String filePref;
  
  /** 
      Rule for execute_rules;
  */
  private Rule r1; 

  public ControlImpl(Hashtable aInfos, Vector rules) {
    this.aInfos = aInfos;
    this.rules = rules;
  }

  public ControlImpl() {
    aInfos = new Hashtable();
    rules = new Vector();
  }

  public ControlImpl(String filePref) {
    this();
    this.filePref = filePref;
  }
  
  public synchronized void shutDown() {
     this.running = false;
  }

  public synchronized int getRetryMul() {
    return this.retry_mul;
  }

  public synchronized void setRetryMul(int retry_mul) {
    this.retry_mul = retry_mul;
  }
  
  public synchronized RepeatFreq getRetryTime() {
    return this.retry;
  }

  public synchronized void setRetryTime(RepeatFreq retry) {
    this.retry = retry;
  }

  public synchronized void setMinMinutes(int min) {
    this.minMinutes = min;
  }

  public synchronized int getMinMinutes() {
    return  minMinutes;
  }

  public boolean cmpRules(Rule r1, Rule r2) {
    if (r1 == r2) {
      return true;
    }

    //Global.LogMessage("Control.cmpRules()","First");
    //logRule(r1);
    //Global.LogMessage("Control.cmpRules()","Second");
    //logRule(r2);

    boolean result =  
            (r1.agent.equals(r2.agent) && 
	     r1.when.equals(r2.when)  && 
	     r1.repeat == r2.repeat && 
	     r1.frequency.every == r2.frequency.every &&
             r1.frequency.unit.value() == r2.frequency.unit.value()
	    );
    if (! result) {
      //Global.LogMessage("Ccmpr", "not result");
      return false;
    } else {
      if (r1.filelists.length != r2.filelists.length) {
        //Global.LogMessage("Ccmpr", "differs");
        return false;
      } else {
        for (int i = 0; i < r1.filelists.length; i++) {
	  if ( ! r1.filelists[i].equals(r2.filelists[i])) {
            //Global.LogMessage("Ccmpr", "flists");
	    return false;
	  } 
	}
	return true;
      }
    }
  }
  public synchronized void addRule(Rule rul, String password) 
                                               throws InvalidPWException {
    if ( rul == null) {
      return;
    }

    if ( checkPassword(rul.agent, password) ) { 
      boolean was_there=false;
      Enumeration renum=rules.elements();
      while (renum.hasMoreElements()) {
	      if (cmpRules(rul,(Rule) renum.nextElement())) {
           was_there=true;
           break;
         }
      }

      if (! was_there) {
        rules.addElement(rul); 
        saveState();
      } else {
        Global.LogMessage("Control.addRule()", "Rule was already there."); 
      }
    }
  }

  public synchronized void removeRule(Rule rul, String password) 
                                            throws InvalidPWException {
    boolean removed = false;
    if ( rul == null) {
      return;
    }

    if ( checkPassword(rul.agent, password) ) { 
      Enumeration renum=((Vector)rules.clone()).elements();
      while (renum.hasMoreElements()) {
	    Rule last=(Rule) renum.nextElement();
	    if (cmpRules(rul, last)) {
              removed = true;
	      rules.removeElement(last); 
             // prozatim zadne break; - hypoteticky by se mohlo objevit vicekrat
            }
      }
      if (! removed) {
        Global.LogMessage("Control.removeRule()", "No matching rule found");
        logRule(rul);
      }
      saveState();
    }
  }
  
     
  protected synchronized boolean checkPassword(String agentname, 
                                                         String password)
                                            throws InvalidPWException {
    Agent ag;
    try {
        ag=Global.GetAgent(agentname); 
        if (ag == null) {
          Global.LogMessage("Control.checkPassword()", "Agent" + agentname + 
                                                       " was not found"      );
          return false;
        }
      } catch (NotInitializedException e) {
        Global.LogMessage("Control.checkPassword()",
                                    "Naming Service not initialized...");
        return false;
      }
      if (ag.checkPassword(password)) {
        aInfos.put(agentname, password);    
        return true;
      } else {
        return false;
      } 
   }

   public synchronized boolean setPassword(String agentname, String password) 
                                             throws InvalidPWException {
      boolean retval = checkPassword(agentname, password);
      saveState();
      return retval;
   }
   
   public Rule [] getRules(String agentname) {
     if (agentname == null) {
       return null;
     }

     Vector matching = new Vector();
     Enumeration erules = rules.elements();
     while(erules.hasMoreElements()) {
       Rule rul = (Rule) erules.nextElement();
       if (rul.agent.equals(agentname)) {
         matching.addElement(rul); 
       }
     }
     Rule [] retdata = new Rule [matching.size()];
     for (int i=0; i < matching.size(); i++) {
        retdata[i] = (Rule) matching.elementAt(i);
     }      
     return retdata;
   }   

   protected static int getRepeatMinutes(RepeatFreq freq) {
     int base = freq.every;
     if (freq.unit == Freq.Minutes) {
     } else if (freq.unit == Freq.Hours) {
       base *= 60;
     } else if (freq.unit == Freq.Days) {
       base *= 60 * 24;
     } else if (freq.unit == Freq.Weeks) {
       base *= 60 * 24 * 7;
     } else if (freq.unit == Freq.Months) {
       base *= 60 * 24 * 7 * 30; // +/- zatim 30 dni
     } else {
       base = 0;
     }
     return base;
   }
   
   protected String reschedule (String when, RepeatFreq freq) {
     int base = getRepeatMinutes(freq);

      if (base < minMinutes) {
        base = minMinutes; 
      }

      Date newdate = new Date();

      // Originally, this was counted from previous time
      
      //try {
      //  newdate.setTime(DateUtil.toDate(when).getTime() + base * 60000);
      //} catch (java.text.ParseException e) {
      //  Global.LogMessage("Control.reschedule()","Invalid date: "+when);
      //  return "!Invalid!";
      //}
        
      // However, we will calculate it from NOW 
      newdate.setTime(newdate.getTime() + base * 60000);
      return DateUtil.toString(newdate);  
   }

   public void reschedule(Rule rul) {
     if (rul.repeat) {
       rul.when = reschedule(rul.when, rul.frequency); 
     } else {
       rul.when = "!Invalid!"; 
     }
   }

   protected Vector merge_rules(Vector orig) {
     Hashtable named = new Hashtable(), helper = new Hashtable();
     Vector merged = new Vector();
     Enumeration elems = orig.elements();
     while (elems.hasMoreElements()) {
       Rule rul = (Rule) elems.nextElement(), rprev;
       if ((rprev=(Rule)named.get(rul.agent)) == null) {
         named.put(rul.agent, rul);
       } else {
         rprev.repeat = false; // Only-once rule
         helper.clear();

         for (int i=0; i < rprev.filelists.length; i++) {
           helper.put(rprev.filelists[i], rprev.filelists[i]);
         }  
         for (int i=0; i < rul.filelists.length; i++) {
           helper.put(rul.filelists[i], rul.filelists[i]);
         }  
         
         String flists [] = new String [helper.size()];
         Enumeration en = helper.elements();
         int i = 0;
         while ( en.hasMoreElements()) {
           flists[i++] = (String) en.nextElement();
         }
         rprev.filelists = flists;         
       }
     }
     elems = named.elements();
     while (elems.hasMoreElements()) {
       merged.addElement(elems.nextElement());
     }
     //logRules(merged);
     return merged;
   }
   
   public synchronized Vector check_rules() {
     Vector hlp = new Vector();
     Enumeration en = ((Vector)rules.clone()).elements();
     Date now = new Date();

     while (en.hasMoreElements()) {
       Rule elem = (Rule) en.nextElement();
       try { 
         if ( ! now.before(DateUtil.toDate(elem.when))) {
           // Cas zalohovat (after() by nemuselo fungovat presne na minutu)
           // Obchazime clone !
           if ( elem.repeat) {
             hlp.addElement(
                            new Rule(elem.agent, elem.repeat, elem.frequency, 
                                     elem.when, elem.filelists)
             );
             reschedule(elem);
           } else {
             rules.removeElement(elem); 
             hlp.addElement(elem);
           } 
         }
       } catch (java.text.ParseException e) {
         Global.LogMessage("Control.checkRules()","Invalid date: "+elem.when);
       }
     }

     if ( hlp.size() > 0) {
       // TBD Merging of backups on same host
       return merge_rules(hlp);
     } else {
       return null;
     }
   }

   public void execute_rules(Vector vec) {
     Enumeration er = vec.elements();
     while (er.hasMoreElements()) {
       r1 = ((Rule) er.nextElement());
       new Thread(
         new Runnable() {
           public void run() {
             final Rule rul = r1;
             String sid = sidgen.genID(rul.agent);
             try {
               Agent ag=Global.GetAgent(rul.agent);  
               Global.LogMessage("Control", "Backup " + sid + " on " 
                                            + rul.agent + " started."
               ); 

               ag.startSession(aInfos.get(rul.agent).toString(), sid);
               Global.GetStorage().backupFiles(ag, sid, rul.filelists);

               Global.LogMessage("Control", "Backup " + sid + " on " 
                                            + rul.agent + " finished."
               ); 
             } catch (Exception e) {
               Global.LogMessage("Control.ExecuteRule()",
                                 "Can\'t exec rule" + sid +" on "+ rul.agent 
                                  + ' ' + e
               ); 
               if (rul.repeat) {
                 // Repeating rule
                 if (getRepeatMinutes(rul.frequency) > 
                                     retry_mul * getRepeatMinutes(retry)) {
                   // the backup will be executed later then retry_mul *
                   // retry_time
                   rul.repeat = false;
                   rul.when=reschedule(rul.when, retry);
                   rules.addElement(rul);
                 } else {
                   Global.LogMessage("Control.ExecuteRule()", 
                                     "Failed rule ignored on " + rul.agent);
                   return;
                 }
               } else {
                 // Only-once, or already failed rule
                 rul.when=reschedule(rul.when, retry);
               }
               Global.LogMessage("Control.ExecuteRule()", 
                                 "Failed rule rescheduled on " + rul.agent);
             }
           } 
         }
       ).start();          
     }
   }
   
   public void getRulesState(OutputStream out) throws java.io.IOException {
      new ObjectOutputStream(out).writeObject(rules);
   } 

   public void getAInfoState(OutputStream out) throws java.io.IOException {
      new ObjectOutputStream(out).writeObject(aInfos);
   } 

   public void setRulesState(InputStream in) {
      try {
        Vector rul;
        rul = (Vector) new ObjectInputStream(in).readObject();
        rules=rul;
      } catch (Exception e) {
        Global.LogMessage("Control.setRulesState()", 
                          "Can\'t read from "+ in + " because of " +e);
      }                                                   
   } 
   public void setAInfoState(InputStream in) {
      try {
        Hashtable ai;
        ai=(Hashtable) new ObjectInputStream(in).readObject();
        aInfos=ai;
      } catch (Exception e) {
        Global.LogMessage("Control.setAInfoState()", 
                          "Can\'t read from "+ in + " because of " +e);
      }                                                   
   } 
   
   public synchronized void saveState() {
     try {
       saveState(filePref);
     } catch (IOException e) {
       Global.LogMessage("Control.saveState()",
                         "Can\'t backup current state: " + e.getMessage());
     }  
   }
   
   public synchronized void saveState(String pref) throws IOException {
     if ( pref != null) {
       saveState(new FileOutputStream(pref+"ai"),   
                 new FileOutputStream(pref+"rules") );   
     }
   }
   
   public synchronized void saveState(OutputStream o_ai, OutputStream o_rules)
                                                            throws IOException {
     getAInfoState(o_ai);
     getRulesState(o_rules);
   }                                                            
   
   public synchronized void loadState() {
     try {
       loadState(filePref);
     } catch (IOException e) {
       Global.LogMessage("Control.loadState()", 
                         "Can\'t load current state: " + e.getMessage());  
     }  
   }
   
   public synchronized void loadState(String pref) throws IOException {
     if ( pref != null) {
       loadState(new FileInputStream(pref+"ai"),   
                 new FileInputStream(pref+"rules") );   
     }
   }
   
   public synchronized void loadState(InputStream i_ai, InputStream i_rules)
                                                            throws IOException {
     setAInfoState(i_ai);
     setRulesState(i_rules);
   }                                                            
   
   public synchronized void logRules() {
     logRules(this.rules);
   }
   
   public synchronized void logRule(Rule rul) {
	 String [] units = {"min.", "hour(s)", "day(s)", "week(s)", "month(s)"};
         Global.LogMessage("Control.logRule()", 
           rul.agent + ' ' + ((rul.repeat)?"rep":"") + " every " +
           rul.frequency.every + " " + units[rul.frequency.unit.value()] + 
           " at " + rul.when
         );
	 for (int i=0; i < rul.filelists.length; i++) {
	   Global.LogMessage("Control.logRule()", 
	                     "FileMask: " + rul.filelists[i]);
	 }		     
   }
   public synchronized void logRules(Vector rules) {
      Enumeration ren = rules.elements();
      while ( ren.hasMoreElements() ) {
         logRule( (Rule) ren.nextElement());
      }
   }
   
   public void run () {
     if (filePref != null) {
       loadState();
     }
     //logRules();
     while ( running ) {
       Vector toGo = check_rules();
       if ( toGo != null ) {
         execute_rules(toGo);
         saveState();
         //logRules();
       }
       try {
         Thread.sleep(SLEEP); // One minute sleep
       } catch (InterruptedException e) {
       }
     } 
   }
}
