import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import Backup.*;
import Backup.RepeatFreqPackage.*;
import java.text.*;


public class ExpireFrame extends JFrame {
  public static String DATE_FORMAT = "yyyyMMdd";
  public static String TIME_FORMAT = "HHmm";

  public static String INIT_STRING="000001010000";
  String toDate = INIT_STRING;
  JPanel contentPane;
  GUI gui=null;
  RulesListPanel rlPanel=null;
  Locale locale=Locale.GERMANY;

  JTextField dateField = new JTextField();
  JTextField timeField = new JTextField();
  JLabel dateLabel = new JLabel();
  JLabel timeLabel = new JLabel();
  JButton okButton = new JButton();
  JButton stornoButton = new JButton();
  JLabel errorLabel = new JLabel();

  public ExpireFrame() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public String getToDate(){
    return toDate;
  }

  
  public ExpireFrame(GUI gui, RulesListPanel rlPanel) {
    this.gui=gui;
    this.rlPanel=rlPanel;

    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public ExpireFrame(GUI gui,RulesListPanel rlPanel, Locale locale) {
    this.gui=gui;
    this.rlPanel=rlPanel;
    this.locale=locale;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }


  private void setDates() {
    Date date;
    if (toDate == null || toDate.compareTo("")==0){
      date = new Date();
     } else {
      try{
        date=DateUtil.toDate(toDate);
      } catch(java.text.ParseException e) {
        System.err.println("Bad DateTime"+e);
        date=new Date();
      }
    }
    dateField.setText(DateFormat.getDateInstance(
        DateFormat.MEDIUM,locale).format(date));
    timeField.setText(DateFormat.getTimeInstance(
        DateFormat.SHORT,locale).format(date));
  }


  private String parseDate(String date, String time) 
		throws java.text.ParseException{
      String s;
      Date d;
 
      SimpleDateFormat dateFormat = new SimpleDateFormat(DATE_FORMAT);
      SimpleDateFormat timeFormat = new SimpleDateFormat(TIME_FORMAT);
      if(date==""){
        d = new Date();
        s=dateFormat.format(d);
      }
      DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM,locale);
      df.setLenient(false);
      d=df.parse(date);
      Date max=DateFormat.getDateInstance(DateFormat.MEDIUM,locale)
        .parse("31.12.9999");
      if(d.compareTo(max)>0){
        throw new java.text.ParseException("Date is to high",0);
      }
      s=dateFormat.format(d);
      if(time==""){
        d = new Date();
        s=dateFormat.format(d);
      }
      d=DateFormat.getTimeInstance(DateFormat.SHORT,locale)
          .parse(time);
      s=s.concat(timeFormat.format(d));

      return s;
  }



  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      rlPanel.controlBack();
    }
  }

  public void reset(){
    toDate = null;
    setDates();
    errorLabel.setText("");
  }

  private void jbInit() throws Exception {
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);
    this.setSize(new Dimension(350, 125));
    this.setTitle("Delete old backup files");


    dateField.setBounds(new Rectangle(55, 10, 95, 25));
    dateField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
	testDates();
      }
    });

    timeField.setBounds(new Rectangle(230, 10, 95, 25));
    timeField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        testDates();
      }
    });


    dateLabel.setText("Date:");
    dateLabel.setBounds(new Rectangle(10, 10, 40, 25));
    timeLabel.setText("Time:");
    timeLabel.setBounds(new Rectangle(180, 10, 45, 25));
    timeLabel.setHorizontalAlignment(SwingConstants.RIGHT);

    okButton.setText("OK");
    okButton.setBounds(new Rectangle(10, 45, 155, 25));
    okButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        okButtonAction(e);
      }
    });
    stornoButton.setText("Storno");
    stornoButton.setBounds(new Rectangle(170, 45, 155, 25));
    stornoButton.addActionListener(new java.awt.event.ActionListener() {

    public void actionPerformed(ActionEvent e) {
        stornoButtonAction(e);
      }
    });
    setDates();


    errorLabel.setForeground(Color.red);
    errorLabel.setText("");
    errorLabel.setBounds(new Rectangle(10, 75, 290, 25));
    contentPane.add(okButton, null);
    contentPane.add(stornoButton, null);
    contentPane.add(dateLabel, null);
    contentPane.add(dateField, null);
    contentPane.add(timeLabel, null);
    contentPane.add(timeField, null);
    contentPane.add(errorLabel, null);
    setResizable(false);
	
  }

  
  private boolean testDates() {
      try {
        toDate = this.parseDate(dateField.getText(),timeField.getText());
        errorLabel.setText("");
        setDates();
        return true;
      } catch (java.text.ParseException e){
        errorLabel.setText("Bad date or time");
        return false;
      }
  }
  
 
  void okButtonAction(ActionEvent e) {
    if (testDates() == true) {
      gui.expireFiles(toDate);
      rlPanel.controlBack();
    }
  }

  void stornoButtonAction(ActionEvent e) {
    rlPanel.controlBack();
  }
}
