import Backup.*;
import java.io.*;
import java.util.zip.Checksum;

public class FileChunkerImpl extends FileChunkerPOA {
   
   protected static int MIN_CHUNKSIZE = 1024;  // V potomkovi lze zmenit
   protected static int DEF_CHUNKSIZE = 65536; // Default 64k
   
   protected InputStream fdata;
   protected byte lastdata [] = null;
   protected int chsize=DEF_CHUNKSIZE;
   protected boolean isEOF = false;
   protected FileInfo fi;
   protected Checksum sumator;
   
   /* Static methods */
   
   /** 
        Sets the default ChunkSize, not alowing to set it below
        minimal one.
   */
   
   public static void SetDefSize(int size) {
      if ( size > MIN_CHUNKSIZE) {
        DEF_CHUNKSIZE = size;
      } else {
        DEF_CHUNKSIZE = MIN_CHUNKSIZE;
      }
   }

   /**
       Gets default transfer chunksize
   */
   
   public static int GetDefSize() {
     return DEF_CHUNKSIZE;
   }

   /* Constructors */

   public FileChunkerImpl(String filename) throws IOException {
      super();
      initSumator();
      File f = new File(filename);
      this.fi = new FileInfo(filename,(int)f.length(), ""); //nezname CRC 
      fdata = new FileInputStream(filename);
   }
   
   public FileChunkerImpl(InputStream fdata, FileInfo fi) {
     super();
     initSumator();
     this.fi = fi;
     this.fdata=fdata;
   }
  
   /* Methods */
   public FileInfo fInfo() {
     return fi; // Nepujde zmenit zvenci ?
   }
   
   protected void initSumator() {
     if (sumator == null) {
       sumator = new java.util.zip.CRC32();
     } else {
       sumator.reset();
     }
   }

  /* 
     Updates checksum
   */
  protected void updateSum () {
    long Svalue=sumator.getValue(); // Nastaveni kontrolniho souctu
    fi.Sum = "CRC32:" + Long.toString(Svalue >> 4, 16) + 
             Long.toString(Svalue & 0xf , 0xf); // a jeho zapis do retezce
  }  
   /**
       Sets current transfer chunksize
   */
   
   public synchronized void chunksize(int size) {
     this.chsize = size;
   }
   
   /**
       Gets current transfer chunksize
   */
   public synchronized int chunksize () {
     return chsize;
   }  
    
   public synchronized boolean hasMoreChunks() {
     return (! isEOF);
   }
   
   public synchronized byte [] nextChunk() 
                                 throws BeyondLastException, FileException {
     if (isEOF) {
       Global.LogMessage("FileChunker.nextChunk()",
                         "Beyond EOF when starting nextChunk() on " + fi.name );
       throw new BeyondLastException();
     }
     
     if ((lastdata == null) || (chsize != lastdata.length)) {
       lastdata = new byte [chsize];
     }
     
     try {
       int wasRead;
       if ( (wasRead=fdata.read(lastdata)) != -1) {
         sumator.update(lastdata, 0, wasRead); // Uprava CRC 
         if (wasRead < chsize) {
           byte [] newdata = new byte [wasRead];

           for (int i = 0; i < wasRead; i++) {
             newdata[i] = lastdata[i];
           }
           
           lastdata=newdata;
           updateSum();
         }
       } else {
         isEOF = true;
         lastdata=new byte[0];
         updateSum();
       }
     } catch (Exception e) {
       Global.LogMessage("FileChunker.nextChunk()",
		         "Read of file " +fi.name+ " failed: " + e); 
       throw new FileException();
     }
     return lastdata;
   }

   public synchronized byte [] retryChunk() 
                                 throws BeforeFirstException {
     if (lastdata == null) {
       Global.LogMessage("FileChunker.retryChunk()",
              "Not read anything when starting retryChunk() on " + fi.name );
       throw new BeforeFirstException();
     }
     return lastdata;
   }
}
