import java.io.*;
import java.util.*;
import java.lang.*;

public class FileMaskExpander{

  protected Vector v;
  protected String mask;

/** 
 * Hleda obsah adresare popripade jen vrati jmeno souboru
 * 
 * @param name jmeno souboru nebo adresare
 * @return vraci jmeno souboru je-li name souborem
 *         vraci jmena souboru a podadresaru (rekurzivne celou hierarchii)
 *               je-li name adresarem
 */
String[] getChilds(String name){
  File f = new File(name);
  String ret[];
  if(!f.exists()){
    return new String[0];
  }
  if(f.isFile()){
    ret = new String[1];
    ret[0]=f.getAbsolutePath();
    return ret;
  }
  if(f.isDirectory()){
    Vector allChilds = new Vector();
    File childs[] = f.listFiles();
    if (childs == null) {
      return new String[0];
    }
    for(int i = 0; i<childs.length; i++) {
      String oneFileChilds[] = getChilds(childs[i].getAbsolutePath());
      if (oneFileChilds == null) {
        continue;
      }
      for(int j = 0; j<oneFileChilds.length; j++){
        allChilds.addElement(oneFileChilds[j]);
      }
    }
    ret = new String[allChilds.size()];
    for(int i=0;i<allChilds.size();i++){
      ret[i]=(String) allChilds.elementAt(i);
    }
    return ret;
  }
  else {
    return new String[0];
  }
}

/**
 * Vrati soubory a adresare (i s podaresari a soubory) odpovidajici masce
 */
private String[] expandOneMask(String mask) { 
  int ind = mask.lastIndexOf('*');
  if(ind == -1) {		//Nejsou '*', zpracuji masku jako soubor
    return getChilds(mask);
  }
  else {
    String ret[] = new String[0];
    File tf= new File(mask);
    String adresar = tf.getAbsolutePath();
    if(!adresar.equals(mask)) {		//Znamena, ze neslo o absolutni cestu
      return ret;
    }
    parseMask(mask);    
    ind = adresar.lastIndexOf(File.separatorChar);
    if(ind != -1) {
      Vector all=new Vector();
      String[] one;
      File f[];
      adresar = adresar.substring(0,ind+1);
      tf = new File(adresar);
      f=tf.listFiles();     
      if(f == null) {
        return ret;
      }
      for(int i=0;i<f.length;i++){
	if(correspond(f[i].getAbsolutePath())){
          all.addElement(getChilds(f[i].getAbsolutePath()));
        }
      }
      int fileCount = 0;
      for(int i = 0;i<all.size();i++){
        fileCount+=((String[])all.elementAt(i)).length;
      }
      ret = new String[fileCount];
      int k = 0;
      for(int i = 0;i<all.size();i++){
        String[] prvek = (String[])all.elementAt(i);
        for(int j=0;j<prvek.length;j++){
          ret[k]=prvek[j];
          k++;
        }
      }
      
    }
    return ret;
  }
}
/**
 * Rozdeli masku na casti mezi znaky '*' a tyto casti ulozi do instancni 
 * promenne v jako vektor retezcu
 */
private void parseMask(String mask){
  v = new Vector();
  int ind;
  int oldind;
  String s;
  ind = mask.indexOf('*');
  this.mask = mask;
  if (ind != -1) {
    if(ind > 0) {
      s = mask.substring(0,ind);
      v.addElement(s);
    }
    while(ind != -1){
      if(ind != mask.length()-1){
        oldind = ind;
        ind = mask.indexOf('*',ind+1);
        if (ind != -1) {
          v.addElement(mask.substring(oldind+1,ind));
        } else {
          v.addElement(mask.substring(oldind+1,mask.length()));
        }
      }
      else {
        break;
      }
    }
  }
  else {
    v.addElement(mask);
  }
}

/**
 * Testuje, zda jmeno souboru odpovida masce nastavene v instancni promenne
 *
 * @param name jmeno souboru, ktery se porovnava s maskou
 * @return true - jmeno souboru odpovida masce
 */
private boolean correspond(String name){
  int pos = 0;
  for(int i=0;i<v.size();i++){
    pos = name.indexOf((String)v.elementAt(i),pos);
    if(pos == -1){
      return false;
    }
    pos+=((String)v.elementAt(i)).length();
  }
  if(mask.charAt(0)!='*') {
    if(!name.startsWith((String)v.elementAt(0))) {
      return false;
    }
  }
  if(mask.charAt(mask.length()-1)!='*') {
    if(!name.endsWith((String)v.elementAt(v.size()-1))) {
      return false;
    }
  }
  return true;
}
 
 /**
  * Nejde vsechny soubory odpovidajici maskam
  *
  * @param masks pole masek, podle kterych se vyhledavaji soubory
  * @return Vector nazvu souboru odpovidajicich maskam
  */
 public Vector expandMask(String[] masks) {
    Vector unmasked = new Vector();
    for (int i=0; i < masks.length; i++) {
      String [] um=expandOneMask(masks[i]);
      for (int j=0; j < um.length; j++) {
        unmasked.addElement(um[j]);
      }
    }
    return unmasked;
  }

/*
public static void main(String argv[]){
  FileMaskExpander f = new FileMaskExpander();
  String name = argv[0];
  Vector s;
  s=f.expandMask(argv);
  for(int i=0;i<s.size();i++){
    System.out.println((String)s.elementAt(i));
  }
}*/
}
