import java.util.Vector;
import Backup.*;
import Backup.RepeatFreqPackage.*;
/**
 * Title:        GUI
 * @author Martin Kot
 * @version 1.0
 */


public class GUI {
  RulesListPanel rlPanel = null;
  LoginFrame loginFrame;
  Storage storage = null;
  Agent agent = null;
  Control control = null;
  String agentName = null;
  String password = null;
  String datestr = null;
  SIDUtil sid = new SIDUtil('R');

  public GUI() {
    loginFrame = new LoginFrame(this);
  }

  public void addRule(Rule rule) {
    try {
      boolean result = control._non_existent();
      if (result) {
         try {
           control=Global.GetControl();
         } catch (NotInitializedException e) {
           System.err.println("Control is not initialized.");
           System.exit(0);
         }
      }
    } catch (Exception e) {
      System.err.println("Fatal error in CORBA " + e);
      System.exit(0);
    }
    try {
      control.addRule(rule,password);
    } catch (InvalidPWException e){
      System.err.println("Invalid password.");	
    }
    refresh();
  }

  public void removeRule(Rule rule) {
    try {
      boolean result = control._non_existent();
      if (result) {
         try {
           control=Global.GetControl();
         } catch (NotInitializedException e) {
           System.err.println("Control is not initialized.");
           System.exit(0);
         }
      }
    } catch (Exception e) {
      System.err.println("Fatal error in CORBA " + e);
      System.exit(0);
    }

    try {
      control.removeRule(rule,password);
    } catch (InvalidPWException e){
      System.err.println("Invalid password.");	
    }
    refresh();
  }

  public void refresh() {
    try {
      boolean result = control._non_existent();
      if (result) {
         try {
           control=Global.GetControl();
         } catch (NotInitializedException e) {
           System.err.println("Control is not initialized.");
           System.exit(0);
         }
      }
    } catch (Exception e) {
        System.err.println("Fatal error in CORBA " + e);
        System.exit(0);
    }
    Rule[] rules = control.getRules(agentName);
    rlPanel.refresh(rules);
  }

  public void expireFiles(String toDate){
    try {
      boolean result = storage._non_existent();
      if (result) {
        try {
          storage=Global.GetStorage();
        } catch (NotInitializedException e) {
          System.err.println("Storage is not initialized.");
          System.exit(0);
        }
      }
    } catch (Exception e) {
      System.err.println("Fatal error in CORBA " + e);
      System.exit(0);
    }

    Global.LogMessage("GUI", "Expiring files stored before " + toDate + " from "
                                   + agentName);
    datestr = toDate;
    new Thread(new Runnable() {
                 public void run() {
                   storage.expireFiles(agentName, datestr);
                 }
               }).start();
  }

  public void restoreFiles(RestoreFrame restoreFrame, String[] filemasks, 
  				boolean replace) {
    try {
      boolean result = agent._non_existent();
      if (result) {
        try {
          agent=Global.GetAgent(agentName);
        } catch (NotInitializedException e) {
          System.err.println("Agent "+agentName+" is not initialized.");
          System.exit(0);
        }
      }

      result = storage._non_existent();
      if (result) {
        try {
          storage=Global.GetStorage();
        } catch (NotInitializedException e) {
          System.err.println("Storage is not initialized.");
          System.exit(0);
        }
      }
    } catch (Exception e) {
      System.err.println("Fatal error in CORBA " + e);
      System.exit(0);
    }

    String s = sid.genID(agentName);
    try {
      try {
        agent.startSession(password,s);
      } catch (InvalidPWException e){
        System.err.println("Invalid password");	
      }
      storage.setRestoreInterval(s,restoreFrame.getFromDate(),
      				restoreFrame.getToDate());
      Global.LogMessage("GUI", "Restore " + s + " on "
                                   + agentName + " started.");
									   
      agent.restoreFiles(storage,s, filemasks, replace);
      Global.LogMessage("GUI", "Restore " + s + " on "
                                   + agentName + " finished.");
    } catch (TransferFailedException e){
      Global.LogMessage("GUI", "Restore " + s + " on "
                                   + agentName + " failed.");
      System.err.println("Transfer failed.");	
    }
  }


  public String getAgentName() {
    return agentName;
  }
  
  public void controlBack() {
    password = loginFrame.getPasswd();
    agentName = loginFrame.getName();
    loginFrame.dispose();
    try {
       try {
       agent=Global.GetAgent(agentName);
       control=Global.GetControl();
       storage=Global.GetStorage();
       }catch (NotInitializedException e) {
         System.out.println("\nInit of NameService failed - "
                         +"it may not be running where you specified,\n"
                         +"or you didn't give the argument -ORBservice "
                         +"NameService <loc>,\nwhere <loc> is either IOR or "
                         +"corbaloc::host:port/NameService"                   );         System.exit(1);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    try {	
      if (!agent.checkPassword(password)) {
	throw new Exception ();
      } 	
      else {
        Rule[] rules = control.getRules(agentName);
        rlPanel = new RulesListPanel(this,rules);
        rlPanel.show();
	RefreshGUI refGUI = new RefreshGUI(this);
	new Thread(refGUI).start();
	rlPanel.requestFocus();
      }
    } catch (Exception e) {
      loginFrame = new LoginFrame(this);
      go(); 	
    }	
  }

  public void go(){
    loginFrame.show();
  }



  public static void main(String[] argv) {
    try {
      // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(argv,null);
      if (!Global.Initialize(orb)) {
         System.out.println("\nInit of NameService failed - "
                         +"it may not be running where you specified,\n"
                         +"or you didn't give the argument -ORBservice "
                         +"NameService <loc>,\nwhere <loc> is either IOR or "
                         +"corbaloc::host:port/NameService"                   );         System.exit(1);
      }
      GUI gui = new GUI();
      gui.go();
    }catch(Exception e) {
      e.printStackTrace();
    }
  }
}
