import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import Backup.*;
import Backup.RepeatFreqPackage.*;
import java.text.*;

public class RulePanel extends JFrame {
  public static String DATE_FORMAT = "yyyyMMdd";
  public static String TIME_FORMAT = "HHmm";

  Rule rule;
  JPanel contentPane;
  RulesListPanel rlPanel=null;
  Locale locale=Locale.GERMANY;
  String toDate = null;

  JCheckBox repeatCBox = new JCheckBox();
  JLabel everyLabel = new JLabel();
  JTextField freqField = new JTextField(5);
  Choice frequencyCBox = new Choice();
  JLabel fileMLabel = new JLabel();
  JTextField fileMField = new JTextField();
  JButton replaceMaskButton = new JButton();
  JButton addButton = new JButton();
  JTextField dateField = new JTextField();
  JTextField timeField = new JTextField();
  JLabel dateLabel = new JLabel();
  JLabel timeLabel = new JLabel();
  List masksList = new List();
  JButton delMaskButton = new JButton();
  JButton okButton = new JButton();
  JButton stornoButton = new JButton();
  JLabel errorLabel = new JLabel();

  public RulePanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public RulePanel(RulesListPanel rlPanel, Rule rule) {
    this.rlPanel=rlPanel;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    setValues(rule);
  }

  public RulePanel(RulesListPanel rlPanel, Rule rule, Locale locale) {
    this.rlPanel=rlPanel;
    this.locale=locale;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    setValues(rule);
  }

  public void setEdit(boolean edit){
    repeatCBox.setEnabled(edit);
    freqField.setEnabled(edit);
    frequencyCBox.setEnabled(edit);
    fileMLabel.setEnabled(edit);
    fileMField.setEnabled(edit);
    replaceMaskButton.setEnabled(edit);
    addButton.setEnabled(edit);
    dateField.setEnabled(edit);
    timeField.setEnabled(edit);
//    masksList
    delMaskButton.setEnabled(edit);

  }

  private RepeatFreq parseRepeat() {
    RepeatFreq frequence= new RepeatFreq();
//    if (this.repeatCBox.isSelected()){
      frequence.unit = Freq.from_int(this.frequencyCBox.getSelectedIndex());
      try {
        frequence.every=Short.parseShort(this.freqField.getText());
        if(frequence.every<=0) {
          throw new NumberFormatException("Frequency must be greater then 0");
        }
      }
      catch (NumberFormatException e) {
        if (this.repeatCBox.isSelected()) {
          System.err.println("Bad frequency"+e);
        }
        frequence.every=1;
      }
      return frequence;
  }


  private boolean testDates() {
      try {
        toDate = this.parseDate(dateField.getText(),timeField.getText());
        errorLabel.setText("");
        setDates();
        return true;
      } catch (java.text.ParseException e){
        errorLabel.setText("Bad date or time");
        return false;
      }
  }

  private void setDates() {
    Date date;
    if (toDate == null || toDate.compareTo("")==0){
      date = new Date();
     } else {
      try{
        date=DateUtil.toDate(toDate);
      } catch(java.text.ParseException e) {
        System.err.println("Bad DateTime"+e);
        date=new Date();
      }
    }
    dateField.setText(DateFormat.getDateInstance(
        DateFormat.MEDIUM,locale).format(date));
    timeField.setText(DateFormat.getTimeInstance(
        DateFormat.SHORT,locale).format(date));
  }
private String parseDate(String date, String time)
                throws java.text.ParseException{
      String s;
      Date d;

      SimpleDateFormat dateFormat = new SimpleDateFormat(DATE_FORMAT);
      SimpleDateFormat timeFormat = new SimpleDateFormat(TIME_FORMAT);
      if(date==""){
        d = new Date();
        s=dateFormat.format(d);
      }
      DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM,locale);
      df.setLenient(false);
      d=df.parse(date);
      Date max=DateFormat.getDateInstance(DateFormat.MEDIUM,locale)
        .parse("31.12.9999");
      if(d.compareTo(max)>0){
        throw new java.text.ParseException("Date is to high",0);
      }
      s=dateFormat.format(d);
      if(time==""){
        d = new Date();
        s=dateFormat.format(d);
      }
      d=DateFormat.getTimeInstance(DateFormat.SHORT,locale)
          .parse(time);
      s=s.concat(timeFormat.format(d));

      return s;
  }



  public void setValues(Rule rule) {
    if(rule==null){
      this.rule=new Rule("",false,
		new RepeatFreq((short)1,Freq.Minutes),"",new String[0]);
    }
    else {
      this.rule=rule;
    }
    Date date;
    if (this.rule.when.compareTo("")==0) {
      date=new Date();
    } else {
      try{
        date=DateUtil.toDate(this.rule.when);
      } catch(java.text.ParseException e) {
        System.err.println("Bad DateTime in rule"+e);
        date=new Date();
      }
    }
    dateField.setText(DateFormat.getDateInstance(
        DateFormat.MEDIUM,locale).format(date));
    timeField.setText(DateFormat.getTimeInstance(
        DateFormat.SHORT,locale).format(date));

    repeatCBox.setSelected(this.rule.repeat);
    setEnabled();
    if (this.rule.repeat==true) {
      freqField.setText(Short.toString(this.rule.frequency.every));
      frequencyCBox.select(this.rule.frequency.unit.value());
    }
    if(this.rule.filelists.length>0){
      fileMField.setText(this.rule.filelists[0]);
    }
    else {
      fileMField.setText("");
    }
    setListValue(this.rule.filelists);
  }

  private void setListValue(String[] fmseq) {
    masksList.removeAll();
    for( int i=0; i<fmseq.length;i++){
      masksList.add(fmseq[i]);
    }
    if(masksList.getItemCount()>0){
      this.replaceMaskButton.setEnabled(true);
      this.delMaskButton.setEnabled(true);
      masksList.select(0);
    } else {
      this.replaceMaskButton.setEnabled(false);
      this.delMaskButton.setEnabled(false);
    }
  }

  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      rule=null;
      rlPanel.controlBack();
    }
  }

  private void jbInit() throws Exception {
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);
    this.setSize(new Dimension(450, 350));
    this.setTitle("Modify rule");


    repeatCBox.setSelected(true);
    repeatCBox.setText("Repeat");
    repeatCBox.setBounds(new Rectangle(10, 40, 70, 25));
    everyLabel.setText("every");
    everyLabel.setBounds(new Rectangle(110, 40, 55, 25));
    freqField.setText("");
    freqField.setHorizontalAlignment(SwingConstants.RIGHT);
    freqField.setBounds(new Rectangle(170, 40, 50, 25));
    frequencyCBox.setBounds(new Rectangle(235, 40, 95, 25));
    dateField.setBounds(new Rectangle(60, 75, 95, 25));
    dateField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        testDates();
      }
    });

    timeField.setBounds(new Rectangle(220, 75, 70, 25));
    timeField.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusLost(FocusEvent e) {
        testDates();
      }
    });

    dateLabel.setText("Date:");
    dateLabel.setBounds(new Rectangle(10, 75, 40, 25));
    timeLabel.setText("Time:");
    timeLabel.setBounds(new Rectangle(170, 75, 45, 25));
    timeLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    fileMLabel.setText("File mask:");
    fileMLabel.setBounds(new Rectangle(10, 110, 80, 25));
    fileMField.setBounds(new Rectangle(100, 110, 270, 25));
    fileMField.addFocusListener(new java.awt.event.FocusAdapter() {

      public void focusGained(FocusEvent e) {
        fileMFieldFocusGained(e);
      }
    });
    this.addFocusListener(new java.awt.event.FocusAdapter() {
      public void focusGained(FocusEvent e) {
        fileMField.requestFocus();
      }
    });
    masksList.setBounds(new Rectangle(10, 145, 270, 120));
    masksList.addItemListener(new java.awt.event.ItemListener() {

      public void itemStateChanged(ItemEvent e) {
        masksListStateChanged(e);
      }
    });
    delMaskButton.setText("Delete mask");
    delMaskButton.setBounds(new Rectangle(290, 145, 145, 25));
    delMaskButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        delMaskButtonAction(e);
      }
    });
    replaceMaskButton.setText("Replace mask");
    replaceMaskButton.setBounds(new Rectangle(290, 175, 145, 25));
    replaceMaskButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        replaceMaskButtonAction(e);
      }
    });
    addButton.setText("Add mask");
    addButton.setBounds(new Rectangle(290, 205, 145, 25));
    addButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        addButtonAction(e);
      }
    });
    okButton.setText("OK");
    okButton.setBounds(new Rectangle(60, 285, 145, 25));
    okButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        okButtonAction(e);
      }
    });
    stornoButton.setText("Storno");
    stornoButton.setBounds(new Rectangle(240, 285, 145, 25));
    stornoButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        stornoButtonAction(e);
      }
    });

    repeatCBox.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand() == "Repeat"){
          setEnabled();
        }
      }
    });
    errorLabel.setForeground(Color.red);
    errorLabel.setText("");
    errorLabel.setBounds(new Rectangle(10, 10, 290, 25));
  

    for(int i=0;i<RulesListPanel.UNIT_TEXTS.length;i++) {
      frequencyCBox.add(RulesListPanel.UNIT_TEXTS[i]);
    }
    contentPane.add(repeatCBox, null);
    contentPane.add(freqField, null);
    contentPane.add(frequencyCBox, null);
    contentPane.add(fileMField, null);
    contentPane.add(fileMLabel, null);
    contentPane.add(dateField, null);
    contentPane.add(timeField, null);
    contentPane.add(dateLabel, null);
    contentPane.add(timeLabel, null);
    contentPane.add(masksList, null);
    contentPane.add(replaceMaskButton, null);
    contentPane.add(delMaskButton, null);
    contentPane.add(everyLabel, null);
    contentPane.add(addButton, null);
    contentPane.add(okButton, null);
    contentPane.add(stornoButton, null);
    contentPane.add(errorLabel, null);
    setResizable(false);

  }

  private void setEnabled(){
    if(repeatCBox.isSelected() == true){
      freqField.setEnabled(true);
      frequencyCBox.setEnabled(true);
      if(rule.frequency==null) {
        freqField.setText("1");
        frequencyCBox.select(0);
      } else {
        freqField.setText(Short.toString(rule.frequency.every));
        frequencyCBox.select(rule.frequency.unit.value());
      }
    }
    else {
      freqField.setText("");
      freqField.setEnabled(false);
      frequencyCBox.setEnabled(false);
    }

  }


  void addButtonAction(ActionEvent e) {
    String fileMF = fileMField.getText();
    if(fileMF !="" && !fileMF.equals("") && fileMF != null){
      this.replaceMaskButton.setEnabled(true);
      this.delMaskButton.setEnabled(true);
      String[] newList = new String[rule.filelists.length+1];
      for(int i=0;i<rule.filelists.length;i++) {
        newList[i]=rule.filelists[i];
      }
      newList[rule.filelists.length]=fileMField.getText();
      rule.filelists=newList;
      masksList.add(fileMField.getText());
      masksList.select(rule.filelists.length-1);
    }
    if(this.masksList.getItemCount()==0) {
      this.replaceMaskButton.setEnabled(false);
      this.delMaskButton.setEnabled(false);
    }
  }

  void masksListStateChanged(ItemEvent e) {
    fileMField.setText(rule.filelists[masksList.getSelectedIndex()]);
    fileMField.setEditable(true);
  }

  void replaceMaskButtonAction(ActionEvent e) {
    int selected = masksList.getSelectedIndex();
    if(fileMField.getText()!=""){
      rule.filelists[selected]=fileMField.getText();
      masksList.replaceItem(fileMField.getText(),selected);
      masksList.select(selected);
    }
  }

  void delMaskButtonAction(ActionEvent e) {
    int selected = masksList.getSelectedIndex();
//    if(fileMField.getText()!=""){
      String[] newList = new String[rule.filelists.length-1];
      for(int i=0;i<rule.filelists.length;i++) {
        int j=0;
        if(i!= selected){
          newList[j]=rule.filelists[i];
          j++;
        }
      }
      rule.filelists=newList;
      masksList.remove(selected);
      masksList.select(0);
//    }
    if(this.masksList.getItemCount()==0) {
      this.replaceMaskButton.setEnabled(false);
      this.delMaskButton.setEnabled(false);
    }
  }

  void okButtonAction(ActionEvent ev) {
    try {
      rule.when = parseDate(dateField.getText(),timeField.getText());
    } catch (java.text.ParseException e){
      errorLabel.setText("Bad date or time");
      return;
    }
    rule.frequency= this.parseRepeat();
    rule.repeat=this.repeatCBox.isSelected();
    rlPanel.controlBack();
  }

  void stornoButtonAction(ActionEvent e) {
    rule=null;
    rlPanel.controlBack();
  }

  void fileMFieldFocusGained(FocusEvent e) {
    fileMField.setSelectionStart(0);
    fileMField.setSelectionEnd(fileMField.getText().length());
  }

  public Rule getRule() {
    return this.rule;
  }

}
