import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import Backup.*;
import Backup.RepeatFreqPackage.*;
import java.text.DateFormat;

/**
 * @author Martin Kot
 * @version 1.0
 */

public class RulesListPanel extends JFrame {
    public static int SHORT_MAX_CHARS
      = Short.toString(Short.MAX_VALUE).length();
    public static String[] UNIT_TEXTS= {"Minutes","Hours","Days","Weeks","Months"};

    public static int BUTTON_X = 310;
    public static int BUTTON_WIDTH = 150;
    public static int BUTTON_HIGHT = 25;
    public static int LABEL_HIGHT = 15;
    public static int LABEL_Y = 30;

    public static int NEW_BUTTON = 1;
    public static int EDIT_BUTTON = 2;
    public static int SHOW_BUTTON = 3;
    public static int RESTORE_BUTTON = 4;
    public static int EXPIRE_BUTTON = 5;

    public static String NO_REPEAT_TEXT = "No repeat";

  protected GUI gui = null;
  protected Rule[] rules=null;
  protected RulePanel rulePanel=null;
  protected RestoreFrame restoreFrame=null;
  protected ExpireFrame expireFrame=null;
  protected Locale locale = Locale.GERMANY;
  protected JPanel contentPane;
  protected int lastButton=0;
  protected int position=0;
  protected boolean active=true;
  protected Rule lastRule=null;


  JButton newRuleButton = new JButton();
  JButton delRuleButton = new JButton();
  JButton editRuleButton = new JButton();
  JButton restoreButton = new JButton();
  JButton expireButton = new JButton();
  JLabel dateLabel = new JLabel();
  JLabel timeLabel = new JLabel();
  JLabel freqLabel = new JLabel();
  List rulesList = new List();
  JButton showRuleButton = new JButton();

  public RulesListPanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public RulesListPanel(GUI gui,Rule[] rules) {
    this.rules=rules;
    this.gui = gui;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public RulesListPanel(GUI gui, Rule[] rules, Locale locale) {
    this.rules=rules;
    this.locale=locale;
    this.gui = gui;
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void refresh(Rule[] rules) {
    position =  rulesList.getSelectedIndex();
    if(active == true){
      this.rules=rules;
    }
    setList();
    if (rulesList.getItemCount() > position){ 
      rulesList.select(position);
    }
    rulesList.requestFocus();
  }


  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }


  private void jbInit() throws Exception {
    rulesList.setMultipleMode(false);
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(null);
    this.setSize(new Dimension(485, 265));
    this.setTitle("List of rules");
    rulesList.setBounds(new Rectangle(10, 50, 290, 175));
    newRuleButton.setText("New rule");
    newRuleButton.setBounds(
      new Rectangle(310, 50, 160, 25));
    newRuleButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        newButtonActionPerformed(e);
      }
    });
    restoreButton.setText("Restore");
    restoreButton.setBounds(
      new Rectangle(310, 170, 160, 25));
    restoreButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        restoreButtonActionPerformed(e);
      }
    });

    expireButton.setText("Delete backups");
    expireButton.setBounds(
      new Rectangle(310, 200, 160, 25));
    expireButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        expireButtonActionPerformed(e);
      }
    });
    
    delRuleButton.setText("Delete rule");
    delRuleButton.setBounds(
      new Rectangle(310, 140, 160, 25));
    delRuleButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        delButtonActionPerformed(e);
      }
    });
    editRuleButton.setText("Edit rule");
    editRuleButton.setBounds(
      new Rectangle(310, 110, 160, 25));
    editRuleButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        editButtonActionPerformed(e);
      }
    });
    showRuleButton.setText("Show rule");
    showRuleButton.setBounds(
      new Rectangle(310, 80, 160, 25));
    showRuleButton.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        showButtonActionPerformed(e);
      }
    });
    dateLabel.setText("Date");
    dateLabel.setBounds(new Rectangle(15, 30, 40,
      15));
    timeLabel.setText("Time");
    timeLabel.setBounds(new Rectangle(90, 30, 40,
      15));
    freqLabel.setText("Repeat frequency");
    freqLabel.setBounds(new Rectangle(150, 30, 160,
      15));
    contentPane.add(rulesList, null);
    contentPane.add(dateLabel, null);
    contentPane.add(timeLabel, null);
    contentPane.add(freqLabel, null);
    contentPane.add(editRuleButton, null);
    contentPane.add(newRuleButton, null);
    contentPane.add(delRuleButton, null);
    contentPane.add(restoreButton, null);
    contentPane.add(expireButton, null);
    contentPane.add(showRuleButton, null);
     
    setResizable(false);
    if(rules != null) {
      this.setList();
    }
    if(rulesList.getItemCount()>0){
      rulesList.select(position);
    }
  }

  private void setList() {
    String s;
    rulesList.removeAll();
    for (int i=0;i<rules.length;i++) {
      s=ruleToString(rules[i]);
      rulesList.add(s);
    }
    if(rulesList.getItemCount()==0){
      delRuleButton.setEnabled(false);
      showRuleButton.setEnabled(false);
      editRuleButton.setEnabled(false);
    }
    else {
      delRuleButton.setEnabled(true);
      showRuleButton.setEnabled(true);
      editRuleButton.setEnabled(true);
      rulesList.select(0);
    }
  }

  private String ruleToString(Rule rule) {
    String s,s2;
    StringBuffer s3;
    String when=rule.when;
    try {
      Date date;
      date=DateUtil.toDate(when);
      s=DateFormat.getDateInstance(
            DateFormat.MEDIUM,locale).format(date)+
        "   "+DateFormat.getTimeInstance(
            DateFormat.SHORT,locale).format(date);

    } catch(java.text.ParseException e) {
      System.err.println("Bad DateTime in rule"+e);
      s=new String();

    }
    if (!rule.repeat) {
      s=s+"     "+NO_REPEAT_TEXT;
    }
    else {
      s2=(new Short(rule.frequency.every)).toString();
      s3=new StringBuffer();
      for (int i=0; i<SHORT_MAX_CHARS-s2.length();i++) {
        s3.append(' ');
      }
      s2=s3+s2+" "+UNIT_TEXTS[rule.frequency.unit.value()];
      s=s+"    "+s2;
    }
    return s;
  }

  private void passControl(Rule r, boolean edit) {
    position = rulesList.getSelectedIndex();
    if(rulePanel==null) {
      rulePanel=new RulePanel(this, r,locale);
    }
    rulePanel.setEdit(edit);
    rulePanel.setValues(r);
    this.setEnabled(false);
    active=false;
    this.rulePanel.show();
  }

  public void controlBack(){
    active = true;
    setEnabled(true);
    requestFocus();
    if(lastButton ==EXPIRE_BUTTON) {
      expireFrame.hide();
      gui.refresh();
      return;
    } 
    if(lastButton ==RESTORE_BUTTON) {
      restoreFrame.hide();
      gui.refresh();
    } else {
      rulePanel.hide();
      setList();
      Rule rule = rulePanel.getRule();
      if(lastButton == NEW_BUTTON) {
        if(rule!=null) {
          rule.agent=gui.getAgentName();
          gui.addRule(rule);
        }
      } else if(lastButton == EDIT_BUTTON) {
        if(rule!=null){
          gui.removeRule(lastRule);    
          gui.addRule(rule);
        }
      } else if(lastButton == SHOW_BUTTON) {
        rulesList.select(position);
	gui.refresh();
      }
    }
  }
  void newButtonActionPerformed(ActionEvent e) {
    lastButton=NEW_BUTTON;
    passControl(null,true);
  }

  void restoreButtonActionPerformed(ActionEvent e) {
    lastButton=RESTORE_BUTTON;
    position = rulesList.getSelectedIndex();
    if(restoreFrame==null) {
      restoreFrame=new RestoreFrame(gui,this,locale);
    }
    this.setEnabled(false);
    restoreFrame.reset();
    active=false;
    restoreFrame.show();
  }
  void expireButtonActionPerformed(ActionEvent e) {
    lastButton=EXPIRE_BUTTON;
    position = rulesList.getSelectedIndex();
    if(expireFrame==null) {
      expireFrame=new ExpireFrame(gui,this,locale);
    }
    this.setEnabled(false);
    expireFrame.reset();
    active=false;
    expireFrame.show();
  }

  void showButtonActionPerformed(ActionEvent e) {
    lastButton=SHOW_BUTTON;
    passControl(rules[rulesList.getSelectedIndex()],false);
  }

  void editButtonActionPerformed(ActionEvent e) {
    lastButton=EDIT_BUTTON;
    
    lastRule = rules[rulesList.getSelectedIndex()];
    Rule elem = new Rule(lastRule.agent, lastRule.repeat, lastRule.frequency,
                         lastRule.when, 
			 (String[])lastRule.filelists.clone());
					 
    passControl(elem,true);
  }

  void delButtonActionPerformed(ActionEvent e) {
    position = rulesList.getSelectedIndex();
    gui.removeRule(rules[position]);    
  }
}
