import java.util.Date;
import java.text.ParseException;

public class SIDUtil {
  
  /** 
       Maximal size of random part of the suffix
  */
  protected static int RSIZE_MAX = 10;
  
  public static final char rand_sep = '_';
  
  protected char prefix = '?';
  protected int rsize = RSIZE_MAX;
  
  
  public SIDUtil() {
  }
  
  public SIDUtil(char prefix, int rsize) {
       this(prefix);
       if (rsize > 0) {    // Pocet znaku pro nahodnou cast
         if (rsize <= RSIZE_MAX) {
           this.rsize = rsize;
         } else {
           this.rsize = RSIZE_MAX;
         }
       } else {
         this.rsize = 0;
       }
  }
  
  public SIDUtil(char prefix) {
       this();
       this.prefix = prefix;
  }
  
  /** 
      Setting default maximal size of random part of data, this doesn't affect
      the random part of already created objects
   */   
   
  public static void SetMaxRsize(int rsm) {
    RSIZE_MAX = rsm;
  }
  
  /**
       Returns the default maximal size of random data tail, however, tail may
       be shorter (in case of leading zeros) or longer by 1 char 
       (if random() returns 1.00)
    */
    
  public static int GetMaxRsize() {
    return RSIZE_MAX;
  }
  
  /** 
      Setting maximal size of random part of data
      for this concreete instance
   */   
   
  public void setRsize(int rs) {
    this.rsize = rs;
  }
  
  /**
       Returns the size of random data tail, however, tail may
       be shorter (in case of leading zeros) or longer by 1 char 
       (if random() returns 1.00)
    */
    
  public int getRsize() {
    return this.rsize;
  }
  
  /** 
       Generates Session ID with inserted String (name) 
  */
  
  public String genID(String name) {
    return genID(name, new Date());
  }
  
  /** 
       Generates Session ID with inserted String (name) for specified date 
       string
  */
  
  public String genID(String name, String when) {
    try {
      return genID(name,DateUtil.toDate(when));
    } catch (Exception e) {
      System.err.println("SidUtil.genID(): invalid date string"+e.getMessage());
      return null;
    }
  }

  /** 
       Generates Session ID with inserted String (name) for specified date
  */
  
  public String genID(String name, Date when) {
    String retstr = "" + prefix + DateUtil.toString(when);
    
    if (name != null) { // Retezec, identifikujici hosta, ap.
      retstr += name;
    }
    
    retstr += rand_sep; // Oddelovac nahodne casti
    
    if (rsize > 0) {
      double rand = Math.random();
      rand *= Math.pow(10,rsize);
      retstr += (long) rand;
    }
    return retstr;
  }
  
  /** 
       Generates Session ID without inserted String
  */
  public String genID () {
    return this.genID(null);
  }

  /** 
      Decoded date 
  */
  
  public static Date getDate(String sid) throws ParseException {
    return DateUtil.toDate(sid.substring(1, DateUtil.GetFormatLength()+1 )); 
  }
  
  /** 
      Name of the agent
  */
  
  public static String getName(String sid) {
    return sid.substring(DateUtil.GetFormatLength()+1, 
                               sid.lastIndexOf(rand_sep)); 
  }
  
  /** 
      Name &amp; encoded date (filename for Storage)
  */
  
  public static String getDatedName(String sid) {
    return sid.substring(1, sid.lastIndexOf(rand_sep)); 
  }

  /**
  <code>
  // Sample code  
  public static void main(String args[]) {
     System.out.println("ID('B',0).genID(): " + 
                              new SIDUtil('B').genID()); 
     System.out.println("ID('X',0).genID(args[0]) + getDatedName(): " + 
                        SIDUtil.getDatedName(new SIDUtil('X').genID(args[0]))); 
     System.out.println("ID('',0).genID(\"test\") + getName(): " + 
                           SIDUtil.getName(new SIDUtil().genID("test"))); 
     try {
       System.out.println("ID('R',15) + getDate(): " + 
                              SIDUtil.getDate(new SIDUtil('R', 15).genID())); 
     } catch (ParseException e) {
       System.err.println("Can't parse Date - error " + e);
     }
  }
  </code>
 */
}
