/* Implementace Storage */

import Backup.*;
import java.util.Vector;
import java.io.FilenameFilter;
import java.io.*;
import java.util.Date;
import java.util.zip.*;
import org.omg.PortableServer.*;
import java.util.Hashtable;

public class StorageImpl extends StoragePOA implements FileSender {

  protected String ZipDir = ".";  
  protected POA poa;
  protected Hashtable table;
  protected Vector zipFiles;
  	

  public StorageImpl(POA poa, Hashtable table) {
    this.poa = poa;
    this.table = table;
  }
  
  public StorageImpl() {
    table = new Hashtable();
  }
  

/*  Volano pri zalohovani 
    source -  agent, ktery posila data
    sid - sessionID
    files - maska podle ktere se vybiraji soubory
*/    
 public void backupFiles(Agent source, String sid, String[] files)
		 throws TransferFailedException {
		 
    FileIterator toSave;
    		 
    try {		 
	toSave = source.sendFiles(sid, files); 
    } catch ( IDException e) {
        Global.LogMessage("Storage ", 
	                  "backup failed when calling sendFiles on agent "
	                  + SIDUtil.getName(sid) + " "  + e); 
	throw new TransferFailedException();  //predelat na neco rozumneho
    }
    
    //create new zip file
    
    String zipFilename = SIDUtil.getDatedName(sid);   
    zipFilename = zipFilename.concat(".zip");
    
    ZipOutputStream archiv = null;
    
    try {
      archiv = new ZipOutputStream((OutputStream) new FileOutputStream(zipFilename));
    } catch (Exception e) {
	 Global.LogMessage("Storage ", "zip file creation failed " + e);     
      throw new TransferFailedException();
    }
    
    ZipEntry  entry;         
    
     //save files
  files:while ( toSave.hasMoreFiles() ) {
  
     FileChunker fc;
       
     try {
       fc = toSave.nextFile();  //ask for next file from agent
     } catch (Exception e) { 
         // chybi reseni retries
       try {
         fc = toSave.retryFile();
       } catch (Exception f) {
	 Global.LogMessage("Storage ", "nextFile failed " + e); 
         continue;  
       }
     }
     
    OutputStream tempFile = null;
    String tempName = null;
    
    try {   // create temp file 
       File whatever = File.createTempFile(sid, "TEMP");    
       tempFile = new FileOutputStream(whatever);
       tempName = whatever.getPath();
       
    } catch (Exception e) {
       Global.LogMessage("Storage ", "cannot create temp file " + tempName + 
        " " + e.getMessage());
    }
    
    
    while (fc.hasMoreChunks()) {  //receive all chunks file consist of
       byte [] chunk;
       
       try {
         chunk=fc.nextChunk();		//NextChunk
       } 
       catch (FileException e){
         // chybi reseni retries
         try {
           chunk=fc.retryChunk(); 	//Retry
	 } catch (BeforeFirstException f) {
             Global.LogMessage("Storage ", "chunk exception " +  e);
	     continue;
	 }
       }
       catch (BeyondLastException f) {
          Global.LogMessage("Storage ", "chunk exception " +  f);       
          continue;
       }
       catch (Exception f) {
	 Global.LogMessage("Storage ", "nextChunk failed " + f); 
	 try {
           tempFile.close();           //zlikvidujem pomocny soubor
	   new File(tempName).delete();
         } catch (IOException g) {
	    Global.LogMessage("Storage ", "error when closing temp file " + g);
	 }   
	  
	 continue files;  
       }
       
       try {   //zapis chunku do pomocneho souboru     
         tempFile.write(chunk);
       } catch (IOException e) {
	 Global.LogMessage("Storage ", "can\'t write to temp file "+ tempName  +": " + e); 
	 try {
           tempFile.close();
	   new File(tempName).delete();
	 } catch (IOException g) {
	    Global.LogMessage("Storage ", "error when closing temp file " + g);	 
	 }
	 continue files;  
       }
     }
     
     try {   //prepis dat z pomocneho souboru do zipka
       entry = new ZipEntry(fc.fInfo().name);
       archiv.putNextEntry(entry); 
       tempFile.close();
       byte[] buffer = new byte[65536];  //vyrovnavaci buffer
       
       BufferedInputStream bufStream = new BufferedInputStream((InputStream) new FileInputStream(tempName));
       
       while(true) {
         int count = bufStream.read(buffer, 0, 65535);
	 if (count == -1) {
	    bufStream.close();
	    new File(tempName).delete();
	    archiv.closeEntry();
	    break;
	 }
	 
	 archiv.write(buffer, 0, count); 
       }
	         
     } catch (IOException g) {
        Global.LogMessage("Storage ", "error writing to zip; file " + fc.fInfo().name);
     }    
      
    }    
    try {
      archiv.close();
      toSave._release();
      source.endSession(sid);  //tell to agent that party is over
      
    } catch (Exception io) {
        Global.LogMessage("Storage ", "error while closeing zip or sending end session to agent " + io);    
    }  
    
 }  
 
 /*  Zlikviduje vsechny zip archivy daneho agenta,
     ktere jsou starsi nez udava before
 */     
 public void  expireFiles(String agentName, String before) {
 
   File thisDir = new File(ZipDir);  // kde mame ulozeny zipy ?
   
   final String exDate = before;
   final String exName = agentName;
   
   //interface pro filtrovani jmen souboru
   FilenameFilter filter = new FilenameFilter() {
   
     public boolean accept(File dir, String name) {
     	      
           String temp;
	   
           try {
             temp = name.substring(12);
           } catch (IndexOutOfBoundsException e) {
                return false;
           }
	   
           if (temp.compareTo(exName+".zip") != 0)  //jde o soubor naseho agenta ?
	     return false;
	   
           try {
             temp = name.substring(0,12);
           } catch (IndexOutOfBoundsException e) {
                return false;
           }
           return (temp.compareTo(exDate) < 0); // spravny cas ?
     }
   };
								     
   String[] fileList = thisDir.list(filter);
   
   if (fileList.length > 0) {
     
     for (int i=0; i < fileList.length; i++)  {
       try {
        new File(fileList[i]).delete();
       } catch (Exception e) {
          Global.LogMessage("Storage ", "delete error in expireFiles() " + e);

       }    	
     }
   }
    
 }
 
 /* Najde nejnovejsi zip archiv daneho agenta
 */
 private String findLatestArchiv(String agent) {
 
   File thisDir = new File(ZipDir);
   final String agentName = agent;
   
   
   //filtr na jmena souboru
   FilenameFilter filter = new FilenameFilter() {
     public boolean accept(File dir, String name) {
     	      
           String temp;
           try {
             temp = name.substring(12);
           } catch (IndexOutOfBoundsException e) {
                return false;
           }
           return (temp.compareTo(agentName+".zip") == 0);
     }
   };
								     
   String winner = null;
   String[] fileList = thisDir.list(filter);
   
   if (fileList.length > 0) {
     winner = fileList[0];
     
     for (int i=1; i < fileList.length; i++)  {
        if (winner.compareTo(fileList[i]) < 0 ) {
          winner = fileList[i];
        }
     }
   }
   
   return winner;
}   


 /* Najde vsechny zip archivy daneho agenta
   lezici v casovem rozmezi from to
  */  
 private Vector findArchivesBetween(String agent, String from, String to) {
 
   File thisDir = new File(ZipDir);
   final String agentName = agent;
   final String exFrom = from;
   final String exTo = to;
   
   //filtr na jmena souboru
   FilenameFilter filter = new FilenameFilter() {
     public boolean accept(File dir, String name) {	      
	              // return name.endsWith("Kocour.zip");
           String temp;
           try {
             temp = name.substring(12);
           } catch (IndexOutOfBoundsException e) {
                return false;
           }
	   
           if (temp.compareTo(agentName+".zip") != 0)
	      return false;
	   
           try {
             temp = name.substring(0,12);
           } catch (IndexOutOfBoundsException e) {
                return false;
           }
           if (temp.compareTo(exTo) > 0)
	      return false;
	      
	   return (temp.compareTo(exFrom) >= 0);      
	   
     }
   };
								     
   Vector winner = new Vector();
   String[] fileList = thisDir.list(filter);
   int j = 0;
   
   if (fileList.length > 0) {
     winner.add(0,fileList[0]);
     
     for (int i=1; i < fileList.length; i++)  {
       for ( j=0; j < winner.size(); j++) {
         
        String temp = (String) winner.elementAt(j);       
        if (temp.compareTo(fileList[i]) < 0 ) {
          break;
        }
       }
       winner.add(j, fileList[i]);    
     
     }
   }     
   
   return winner;
 }
            
  /* Otevre zip zrchivy pro cteni
     jmena archivu udava filenames
  */     
  private void openZips(Vector filenames) throws IOException {
  
    zipFiles = new Vector(); 

    for(int i=0; i< filenames.size(); i++) {
      ZipFile temp = new ZipFile((String) filenames.get(i));
      zipFiles.add(temp);
    }

  }


  /*  Metoda volana agentem pri obnovovani
  */
  public FileIterator sendFiles(String sid, String[] files) throws IDException {

     
     Vector aFileNames;
    
     if (table.containsKey(sid)) {  // mame v tabulce specifikovane casove rozmezi ?
       RestoreInfo rInfo = (RestoreInfo) table.get(sid);
       aFileNames = findArchivesBetween(SIDUtil.getName(sid), rInfo.from, rInfo.to);
     } else
     {
        // jinak pouzijeme nejnovejsi archiv
        aFileNames = new Vector();
        String filename = findLatestArchiv(SIDUtil.getName(sid));
        if (filename != null)
	   aFileNames.add(filename); 
     }	     
    
    // Global.LogMessage("Storage.sendFiles()", "Archive name: " + filename);
    
     if (aFileNames.size() > 0) {
     
         StorageImpl stor2; // Upravit 
         stor2 =  new StorageImpl(_default_POA(), this.table);
       
       try {
	 stor2.openZips(aFileNames);
	 
       } catch (Exception e) { 
         Global.LogMessage("Storage.sendFiles() ", "noassign" + e );
         throw new IDException();
       }	         
	 
      // FileMaskExpander fme = new FileMaskExpander(files);
       
       FileIteratorImpl fii = new FileIteratorImpl
                              ( new StorageFileMaskExpander().expandMask(files, stor2.zipFiles),
			        (FileSender) stor2 );
       try {
         return FileIteratorHelper.narrow(
                     _default_POA().servant_to_reference(fii)
                );
       } catch (Exception e) { 
         Global.LogMessage("Storage.sendFiles()", "noref" + e);
       }
     }
     
     try {
       return FileIteratorHelper.narrow(
                     _default_POA().servant_to_reference(
		       new FileIteratorImpl(new Vector())
		     ));
    } catch (Exception e) {
         Global.LogMessage("Storage.sendFiles()", "CORBERR");
	 return null;
    }
 }

 /* Metoda pouzivane agentem pro ziskavani jednotlivych
    souboru od Storage
 */     
 public FileChunker getFile(String filename) throws FileException {
 
    FileChunkerImpl fci;
    try {
           //na zactku filename mame index urcujici
	   //z ktereho zipka budeme soubor tahat
      int hv = filename.indexOf("*");
      
      String sNum = filename.substring(0, hv);
      int zipNum = Integer.parseInt(sNum); 
      String path = filename.substring(hv + 1); 
    
      ZipFile zipFile = (ZipFile) zipFiles.get(zipNum);
      ZipEntry entry = zipFile.getEntry(path);
      
      InputStream zipStream = zipFile.getInputStream(entry);
    
      FileInfo fInfo = new FileInfo(path, (int) entry.getSize(), "");
      
      fci = new FileChunkerImpl(zipStream, fInfo);
      
      return FileChunkerHelper.narrow( poa.servant_to_reference(fci));
    
    } catch (Exception e) {
       Global.LogMessage("Storage.getFile()", "Efile " + e);
       throw new FileException();
    }  
 }
 
 /* Pro dane sessionID nastavi casovy intervel,
    ze ktereho se budou brat zip archivy
  */    
 public void setRestoreInterval(String sid, String from, String to) {
 
    RestoreInfo info = new RestoreInfo(from, to);
    table.put(sid, info);
 
 }       

 /* Volano agentem, aby Storage vedel, ze prenos je u konce
 */
 public synchronized void endSession(String sid) {
 
   if (table.containsKey(sid))
     table.remove(sid);
     
   this.endSession();
 } 
 

 /* Pomocna endSession*/
 private void endSession() {
 
   if ( (zipFiles != null) && (this.zipFiles.size() != 0)) {   // asi neni nutne, zda se ze v
    try {                                                      //v dobe volani jsou uz zavrene
     for(int i=0; i < this.zipFiles.size(); i++) {
        
        ZipFile zfile = (ZipFile) this.zipFiles.get(i);
        zfile.close();
     }	
     } catch (Exception e) {
       Global.LogMessage("Storage.endSession()",  e.getMessage());       
     }
   }
   
 } 
}
